function AFPar = OpenAudioFile (File, AFPar)
% Print information about an audio file, pick up the file parameters. This
% routine can open WAVE, AU, and  headerless (raw) audio files.
%
% AFPar = OpenAudioFile (File)
% AFPar = OpenAudioFile (File, AFPar)  % For headerless files only
% The output structure AFPar is used as input to ReadAudioData to
% read audio data from the file.
%
% The second argument to OpenAudioFile is used to determine if the input
% file is to be considered as headerless. If that argument is not present,
% headerless files will not be accepted. For headerless files, the input
% structure contains information about the file format. For headerless
% files, it is assumed that data is in 16-bit format.
%   AFPar.Sfreq: Sampling frequency (Hz), defaults to 8000
%   AFPar.Nchan: Number of channels, defaults to 1
%   AFPar.Swapb: Data swap information, 'Native', Swap', 'Little-endian',
%                or 'Big-endian', defaults to 'Native'
%   AFPar.Start: Start byte for data
% For the case of reading all of the audio data in one shot, ReadAudioData
% can be used directly (it will call OpenAudioFile itself).
%
% The parameters in the output structure AFPar include
% - AFPar.Sfreq: Sampling frequency (Hz)
% - AFPar.Nsamp: Number of samples (per channel)
% - AFpar.Nchan: Number of channels
% The output structure contains additional fields used to store state
% information for use by ReadAudioData.

% $Id: OpenAudioFile.m,v 1.6 2009/07/11 11:06:20 pkabal Exp $

% Open the file for a peek at the first 4 characters
fid = fopen (File, 'r');

if (fid == -1)
  error ('OpenAudioFile: Cannot open audio file for reading');
end
File = FullName (File);

id = char (fread (fid, 4, 'uchar')');
fclose (fid);

switch id
  case 'RIFF'
    AFPar = OpenWAVEFile (File);
  case '.snd'
    AFPar = OpenAUFile (File);
  otherwise
    if (nargin == 2)
      AFPar = OpenRawFile (File, AFPar);
    else
      error ('OpenAudioFile: Unsupported file type');
    end
end

% Set up a null buffer
AFPar.Nbuff = 0;
AFPar.Buff = [];
AFPar.Offs = 0;

PrintAudioFilePar (AFPar);

% --------------------
function AFPar = OpenWAVEFile (File)

[Size Sfreq NbS opt] = wavread (File, 'size');
AFPar.FType = 'WAVE';
AFPar.Fname = File;
AFPar.Sfreq = Sfreq;
AFPar.Nsamp = Size(1);
AFPar.Nchan = Size(2);
AFPar.DFormat = 'unsupported';
switch opt.fmt.wFormatTag
  case 1
    switch NbS
      case 8
        AFPar.DFormat = 'unsigned8';
      case 16
        AFPar.DFormat = 'integer16';
      case 24
        AFPar.DFormat = 'integer24';
      case 32
        AFPar.DFormat = 'integer32';
    end
  case 2
    switch NbS
      case 32
        AFPar.DFormat = 'float32';
      case 64
        AFPar.DFormat = 'float64';
    end
  case 6
    AFPar.DFormat = 'mu-law8';
  case 7
    AFPar.DFormat = 'A-law8';
end

return

% --------------------
function AFPar = OpenAUFile (File)

[Size Sfreq NbS] = auread (File, 'size');
AFPar.FType = 'AU';
AFPar.Fname = File;
AFPar.Sfreq = Sfreq;
AFPar.Nsamp = Size(1);
AFPar.Nchan = Size(2);

% auread does not return the data format directly
% 8 bits/sample can be mulaw8, A-law8 (not supported), or integer8.
% We assume it is mu-law8.
% Similarly for 32 bits/sample, we assume float32 rather than integer32.
switch NbS
  case 8
    AFPar.DFormat = 'mu-law8';
  case 16
    AFPar.DFormat = 'integer16';
  case 24
    AFPar.DFormat = 'integer24';
  case 32
    AFPar.DFormat = 'float32';
  case 64
    AFPar.DFormat = 'float64';
  otherwise
    AFPar.DFormat = 'unsupported';
end

return

% --------------------
function AFPar = OpenRawFile (File, AFPar)

if (~isfield (AFPar, 'Sfreq'))
   AFPar.Sfreq = 8000;
end
if (~isfield (AFPar, 'Nchan'))
   AFPar.Nchan = 1;
end
if (~isfield (AFPar, 'Swapb'))
   AFPar.Swapb = 'Native';
end
if (~isfield (AFPar, 'Start'))
   AFPar.Start = 0;
end

% rawread only supports 16 bits/sample
Size = AFrawRead (File, 'size', AFPar.Swapb, AFPar.Start, AFPar.Nchan);
AFPar.FType = 'Raw';
AFPar.Fname = File;
AFPar.Nsamp = Size(1);
AFPar.DFormat = 'integer16';

return

% --------------------
function PrintAudioFilePar (AFPar)
% Print audio file parameters

switch AFPar.FType
  case 'WAVE'
    FType = 'WAVE';
  case 'AU'
    FType = 'AU';
  case 'Raw'
    FType = 'Headerless';
  otherwise
    FType = 'unknown';
end

switch AFPar.DFormat
  case 'integer8'
    DFormat = '8-bit integer';
  case 'unsigned8';
    DFormat = 'offset-binary 8-bit integer';
  case 'integer16'
    DFormat = '16-bit integer';
  case 'integer24'
    DFormat = '24-bit integer';
  case 'integer32'
    DFormat = '32-bit integer';
  case 'float32'
    DFormat = '32-bit float';
  case 'float64'
    DFormat = '64-bit float';
  case 'mu-law8'
    DFormat = '8-bit mu-law';
  case 'A-law8'
    DFormat = '8-bit A-law';
  otherwise
    DFormat = 'unknown';
end

fprintf (' %s file: %s\n', FType, AFPar.Fname);
fprintf ('   Number of samples : %d (%.4g s)\n', AFPar.Nsamp, ...
              AFPar.Nsamp / AFPar.Sfreq);
fprintf ('   Sampling frequency: %g Hz\n', AFPar.Sfreq);
fprintf ('   Number of channels: %d (%s)\n', AFPar.Nchan, DFormat);

return
