function [x, Nv, AFPar] = ReadAudioData (AFPar, offs, Ns)
% Read data from an audio file. This routine handles WAVE, AU, and
% headerless (raw) files. This routine maintains an internal buffer to
% make file access more efficient when accessing small chunks of data.
% In addition, the data in the file is treated as if it were preceded
% and followed with zeros.
%
%   x = ReadAudioData (File)
%   x = ReadAudioData (File, offs, Ns)
% This case represents the simple interface for reading data from an audio
% file in one shot. Internally, ReadAudioData calls OpenAudioFile to get
% the file parameters. This simple interface cannot be used for headerless
% files.
% - x:    Audio data
% - offs: Offset from the beginning of the file for the start of data
%         (default zero)
% - Ns:   Number of samples to be read (default to end-of-file)
%
%   [x, Nv, AFPar] = ReadAudioData (AFPar, offs, Ns)
% This case is used for repeated reading of data from a file. The file must
% be first opened with OpenAudioFile. OpenAudioFile returns the structure
% AFPar which is used as input to ReadAudioData.
% - x:  Audio data
% - Nv: Number of samples (not including trailing zeros) returned. This
%       value can be used to determine when the end-of-file occurs.
% - AFPar: Structure from OpenAudioFile (input and output)
% - offs: Offset from the beginning of the file for the start of data
%       (default zero)
% - Ns: Number of samples to be read (default to end-of-file)
%
% When reading small chunks of data, AFPar contains a buffer of samples.
% Future requests are checked whether they can be satisfied from the buffer
% without accessing the file.
%
% Note that the audio data in the file is considered to be preceded and
% followed by zeros. With this convention, the offset can be negative, in
% which case zeros are added in front of the data. Also the read can extend
% beyond the end-of-file, in which case zeros are added at the end of the
% data.

% $Id: ReadAudioData.m,v 1.4 2009/07/11 11:06:42 pkabal Exp $

% Call OpenAudioFile if the first argument is not a structure
if (~isstruct (AFPar))
  File = AFPar;
  AFPar = OpenAudioFile (File);
end

Nsamp = AFPar.Nsamp;
Nchan = AFPar.Nchan;

% Default values for offs and Ns
if (nargin < 2)
  offs = 0;
end
if (nargin < 3)
  Ns = max (0, Nsamp - offs);
end

% Allocate storage and zero the output array
x = zeros (Ns, Nchan);

% Calculate number of zeros before the start of data
Nz = 0;
if (offs < 0)
  Nz = min (-offs, Ns);
  offs = offs + Nz;
end

% Get samples from the file
Nf = min (Ns - Nz, Nsamp - offs);
if (offs >= 0 && Nf > 0)
  if (nargout < 3)
    x(Nz+1:Nz+Nf,1:Nchan) = AF_audio_data (AFPar, offs, Nf);
  else
    [x(Nz+1:Nz+Nf,1:Nchan), AFPar] = AF_audio_data (AFPar, offs, Nf);
  end
end

% Number of samples, excluding zeros added after file data
Nv = Nz + Nf;

%------------------------
function [x, AFPar] = AF_audio_data (AFPar, offs, Ns)

MAXBUFF = 50000;

Rlimits = [offs offs+Ns-1];                         % Request limits
Blimits = [AFPar.Offs AFPar.Offs+AFPar.Nbuff-1];    % Buffer limits
Dlimits = [0 AFPar.Nsamp-1];                        % Data limits

% Error check (should never occur if called properly)
if (Rlimits(1) < Dlimits(1) && Rlimits(2) >= Dlimits(2))
  error ('ReadAudioData: Invalid limits');
end

% Read data directly into the user buffer
% - When the read is for the complete file
% - When the transfer is a large amount such that two successive
%   reads of the same amount would entail going to the file anyway
% - When the output structure is not used 
if (Rlimits(1) == Dlimits(1) && Rlimits(2) == Dlimits(2) || ...
    Ns > 0.5 * MAXBUFF || nargout < 2)
  x = AF_Read_audio_data (AFPar, Rlimits);
  AFPar.Nbuff = 0;
  AFPar.Buff = [];
  AFPar.Offs = Rlimits(1);

% Data available in the buffer
elseif (Rlimits(1) >= Blimits(1) && Rlimits(2) <= Blimits(2))

  Boffs = Rlimits(1)-Blimits(1);
  x = AFPar.Buff(Boffs+1:Boffs+Ns);

% Data has to be refreshed into the buffer
else
  Nchan = AFPar.Nchan;
  Nbuff = min (Dlimits(2)-Rlimits(1)+1, MAXBUFF);
  AFPar.Buff = AF_Read_audio_data (AFPar, [Rlimits(1) Rlimits(1)+Nbuff-1]);
  AFPar.Nbuff = Nbuff;
  AFPar.Offs = Rlimits(1);
  x = AFPar.Buff(1:Ns,1:Nchan);

end

return

%----------------------
function x = AF_Read_audio_data (AFPar, Rlimits)

switch AFPar.FType
    case 'WAVE'
        x = wavread (AFPar.Fname, Rlimits + 1);
    case 'AU'
        x = auread (AFPar.Fname, Rlimits + 1);
    case 'Raw'
        x = AFrawRead (AFPar.Fname, Rlimits + 1, AFPar.Swapb, ...
                     AFPar.Start, AFPar.Nchan);
    otherwise
        error ('ReadAudioData: Unsupported audio file type');
end

return

