function G7231Coder (FNameI, FNameO)
% ITU-T G.723.1 speech coder
% FNameI: Input audio file (.wav, .au, ... )
% FNameO: Output bit stream file (.bit) or output data file (.dat)

% P. Kabal, www.TSP.ECE.McGill.CA

% $Id: G7231Coder.m,v 1.14 2009/07/15 16:25:31 pkabal Exp $

G7231Dir = fileparts (which ('G7231Coder.m'));
addpath (fullfile (G7231Dir, 'ACB'), fullfile (G7231Dir, 'Audio'), ...
         fullfile (G7231Dir, 'BitStream'), ...
         fullfile (G7231Dir, 'Filter'), fullfile (G7231Dir, 'LP'), ...
         fullfile (G7231Dir, 'Misc'), ...
         fullfile (G7231Dir, 'MultiPulse'), ...
         fullfile (G7231Dir, 'Tables'), fullfile (G7231Dir, 'Target'));

% Set up the output file name
if (nargin <= 1)
  [pathstr name] = fileparts (FNameI);
  FNameO = fullfile ('', [name, '.bit']);
end
[pathstr name ext] = fileparts (FNameO);
isBitStream = strcmpi (ext, '.bit');

% Coder parameters
Np = 10;
LSubframe = [60 60 60 60];
CoderPar = SetCoderPar (Np, LSubframe);

HPFilt = CoderPar.HPFilt;
LPpar = CoderPar.LPpar;
LSFpar = CoderPar.LSFpar;
Pitchpar = CoderPar.Pitchpar;
TVpar = CoderPar.TVpar;
SineDetpar = CoderPar.SineDetpar;
MPpar = CoderPar.MPpar;

% Initialize the coder memory
CoderMem = InitCoderMem (CoderPar);

% Initialize the weighted synthesis filter memories
WSynMem0 = WSynInit (length (TVpar.HNWpar.xp));
WSynMem = WSynMem0;

LFrame = sum (LSubframe);

% Open the input file
AFPar = OpenAudioFile (FNameI);

% Initialize the previous frame data
% The frame consists of 3 parts, [lookback Frame lookahead]
% The lookback and lookahead are each half a window long. The total memory
% of the system is lookback+lookahead. New data is read into the end of the
% array.
offs = 0;
xpMem = zeros (LPpar.LMem, 1);

FSt = LPpar.FStart+1;
FEn = FSt + LFrame - 1;
FMode = 1;        % Multipulse
TickTock (LFrame / AFPar.Sfreq, 1, '< Data Time: %d s >\n');
IFr = 0;
while (1)

  % Read a frame of audio data
  [x, Nv, AFPar] = ReadAudioData (AFPar, offs, LFrame);
  if (Nv < LFrame)
    break
  end

  TickTock;
  IFr = IFr + 1;

  % Highpass filter
  [xf, HPFilt.Mem] = filter (HPFilt.b, HPFilt.a, x, HPFilt.Mem);

  % Form the extended data buffer of high-passed signal
  % Concatenate the new values (1 frame) onto the saved values
  xe = [xpMem; xf];

  % LP analysis
  a = LPanalFrame (xe, LPpar);

  %  % Find the open loop pitch estimate
  % Perceptual weighting filter coefficients
   xc = xe(FSt:FEn);	% Current frame samples
   [xt, LOL, WSynCof, TVpar] = GenTarget (xc, a, TVpar);

  % Set up the data memory for the next frame (save LMem values)
  xpMem = xe(end-LPpar.LMem+1:end);

  % Sine detector (from LP parameters)
  [SineDet, SineDetpar.rc] = SineDetector (a, SineDetpar);

  % Quantize the LP parameters (as LSFs)
  % LSFC:  Codes to index the LSF codebooks (one set per frame)
  LSFC = LPxLSFQ (a(:,LPpar.SFRef), CoderMem.lsfQ, LSFpar);
  
  % Inverse quantization of the LSFs
  % Interpolation of the LSFs
  % Convert LSFs to LP coefficients
  % aQI:   Quantized interpolated LP parameters (one per subframe)
  % lsfQ:  Quantized LSF's (saved for differential coding) 
  [aQI, CoderMem.lsfQ] = LSFCxLPI (LSFC, CoderMem.lsfQ, LSFpar);

  NSubframe = length (LSubframe);
  LPrev = NaN;
  j = 0;
  for (i = 1:NSubframe)

    N = LSubframe(i);

    % Set up the weighted synthesis filter coefficients
    % The weighted synthesis filter has three parts
    % 1. Quantized all-pole filter (aQ). These are added here.
    % 2. Formant weighting filter (set up earlier), derived from
    %    the unquantized LP parameters
    % 3. Harmonic weighting filter (set up earlier)
    % Calculate the weighted synthesis filter impulse response
    WSynCof(i).aQ = aQI(:,i);
    wIR = WSyn (eye(N,1), WSynCof(i), WSynMem0);

    % Subtract the zero-input response from the target
    % This uses the state from the previous subframe
    zIR = WSyn (zeros(N,1), WSynCof(i), WSynMem);
    xtz = xt(j+1:j+N) - zIR;

    % Adaptive codebook contribution
    % The ACB searches around the open loop lag for the first and
    % third subframes and around the previous closed loop lag for
    % the second and fourth subframes
    % ACBLC: Lag code for the pitch lag (lag relative to the minimum lag
    %      or relative to the previous lag)
    % ACBbIB: Gain indices, index and codebook index.
    PMode = Pitchpar.PMode(i);
    if (PMode == 1)
      Lx = LOL(i);
    else
      Lx = LPrev;
    end
    [ACBLC(i), ACBbIB(:,i), Pitchpar.Tamepar.E] = ...
                                GetACB (xtz, CoderMem.eMem, wIR, PMode, ...
                                        Lx, SineDet, Pitchpar);

    % Update the target vector with the pitch contribution
    [L(i), b(:,i)] = DecodeACBSF (ACBLC(i), ACBbIB(:,i), PMode, LPrev, ...
                                  Pitchpar); % Excitation contribution
    ep = PitchContrib (N, L(i), b(:,i), CoderMem.eMem, Pitchpar);
    xp = filter (wIR, 1, ep);                % Weighted signal contribution
    xtpz = xtz - xp;

    % Fixed codebook pulse positions
    if (PMode == 1)
      Lx = L(i); % The lags used are: L(1), L(1), L(3), L(3)
    else
      Lx = LPrev;
    end
    Pulseval(i) = MultiPulse (xtpz, wIR, Lx, i, MPpar);   
    LPrev = L(i);

    % Fixed codebook contribution
    em(:,i) = MPContrib (N, Pulseval(i));
    es = ep + em(:,i);

    % Update the filter memories, ignoring the output
    [temp, WSynMem] = WSyn (es, WSynCof(i), WSynMem);
    
    % Update the excitation memory
    CoderMem.eMem = ShiftVector (CoderMem.eMem, ...
                                 ClipSignal (es, CoderPar.Clippar));
    j = j + N;

  end

  % Create the coded data
  if (isBitStream)
    QC(IFr) = CodeStream (FMode, LSFC, ACBLC, ACBbIB, Pulseval, ...
                          Pitchpar, MPpar);
  else
    VC(IFr) = CodeValue (FMode, aQI, L, b, em);
  end

  % Move ahead in the speech
  offs = offs + LFrame;

end

% Write the bitstream or data file
if (isBitStream)
  WriteG7231Stream (FNameO, QC);
else
  save (FNameO, 'VC', '-mat');
  fprintf ('G723.1 data file: %s\n', FullName (FNameO));
end

return

%--------------------
function QC = CodeStream (FMode, LSFC, ACBLC, ACBbIB, Pulseval, ...
                          Pitchpar, MPpar)
% Create the coded bit stream data

% Code the gains for the pitch and the multipulse contributions
CGC = CodeGain (ACBbIB, Pulseval, Pitchpar);
    
% Multipulse grid codes, sign codes, positions codes
[MPGridC, MPPosC, MPSignC] = MPCode (Pulseval, MPpar);

% Save the codes for transmission
QC.FType = FMode - 1;
QC.LSFC = LSFC;
QC.ACBLC = ACBLC;
QC.CGC = CGC;
QC.MPGridC = MPGridC;
QC.MPPosC = MPPosC;
QC.MPSignC = MPSignC;

return

%--------------------
function VC = CodeValue (FMode, a, L, b, em)
% Save the coded values
% FType: Type (MP / ACELP)
% a:   LP coefficients for each subframe
% L:   Pitch lag for each subframe
% b:   Pitch coefficient vectors for each subframe
% em:  Fixed codebook contribution for each subframe

VC.FType = FMode - 1;
VC.a = a;
VC.L = L;
VC.b = b;
VC.em = em;

return
